<?php

/**
 * 20/10/2015
 * @author Victor Novy
 */

namespace Modules\DF\Services;

use \Teknisa\Libs\Util\DateUtil;
use \Doctrine\ORM\EntityManager;
use \Zeedhi\Framework\DB\StoredProcedure\Param;
use \Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use \Estoque\Entities\Lanctoestoq;
use \Estoque\Entities\Movilote;
use \Estoque\Entities\Posestdia;
use \Estoque\Entities\Itlanctoest;
use \Geral\Entities\Produto;
use \Zeedhi\Framework\ORM\DateTime;

class ValidacoesEstoqueService {
    const ENTRADA       = 'E';
    const SAIDA         = 'S';
    const TIPO_CUSTO_PADRAO = '03';
    const ESTOQUE_VAZIO = ' ';

    protected $entityManager;
    private $lanctoestoq;
    protected $errors = array();
    
    public function __construct (EntityManager $entityManager) {
        $this->entityManager = $entityManager;
    }
    
    /* ******************** Consulta o Custo e a Quantidade do Produto ******************** */
    //$filial - cdfilimovi
    //$produto - cdprodesto
    public function consultaCustoQuantidadeProduto(
        $filial, $produto, $dtLancamento, $tipoCusto = self::TIPO_CUSTO_PADRAO,
        $almoxarifado = self::ESTOQUE_VAZIO, $localEstoque = self::ESTOQUE_VAZIO,
        $lote = self::ESTOQUE_VAZIO, $subLote = self::ESTOQUE_VAZIO , $operacao
    ){
        $custoQuantidade = null;
        
        //tipo de custo é retornado na propria pela consulta
        /*
        $operacao = $this->getTipoOperac($operacao);
        $paramFilial = $this->getParamFilial($filial);
         
        $cdtipcustest = $paramFilial["CDTIPCUSTEST"];
        $tpCustoOp = $operacao["CDTIPOCUSTO"];
        $cdprecobase = $operacao["CDPRECOBASE"];

        if(!empty($tpCustoOp)){
            $tipoCusto = $tpCustoOp;
        }else if(!empty($cdtipcustest)){
            $tipoCusto = $cdtipcustest;
        }else{
            $custoQuantidade = array('VRCUSTOPROD' => '0', "ExistsVr" => false, "ExistsTp" => false);
            return $custoQuantidade;
        }
        */
        $custoQuantidade = $this->consultaCustoQuant(
            $filial, $tipoCusto, $produto, $almoxarifado, $localEstoque, $lote, $subLote, $dtLancamento
        ); 
        if ($custoQuantidade["VRCUSTOPROD"] == 0){
            $custoQuantidade = array('VRCUSTOPROD' => 0, "ExistsVr" => false, "ExistsTp" => true);
            return $custoQuantidade;
        }

        return array('VRCUSTOPROD' => $custoQuantidade['VRCUSTOPROD'], "ExistsVr" => true, "ExistsTp" => true);
    }
    
    public function getParamFilial($cdfilial){
        $sql = <<<SQL
        SELECT * FROM PARAMFILIAL WHERE CDFILIAL = :CDFILIAL
SQL;

        $params = array(
            "CDFILIAL" => $cdfilial
        );

        try{
            return $this->entityManager->getConnection()->fetchAssoc($sql, $params);
        } catch(\Exception $e) {
            throw $e;
        }
        
    }
    
    public function getTipoOperac($operacao){
        $sql = <<<SQL
        SELECT * FROM TIPOOPERAC WHERE IDENTRSAIDOP = 'S' AND CDTIPOOPER = :CDTIPOOPER
SQL;

        $params = array(
            "CDTIPOOPER" => $operacao
        );

        try{
            return $this->entityManager->getConnection()->fetchAssoc($sql, $params);
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function consultaCustoQuant(
        $filial, $tipoCusto, $produto, $almoxarifado, $localEstoque, $lote, $subLote, $dtLancamento
    )
    {
        
        $sql = <<<SQL
        SELECT 
        CUSTO_PRODUTO_LCEST(
                            :P_CDFILIAL,
                            P.CDPRODESTO, 
                            (SELECT NVL(TO_NUMBER(CDTIPCUSTEST),3) FROM PARAMFILIAL WHERE CDFILIAL = :P_CDFILIAL), 
                            TO_DATE(:P_DTLANC),
                            :P_DTLANC,
                            ' ', 
                            ' ', 
                            ' ', 
                            ' '
                        ) * NVL(P.VRFATOCONV, 1) AS VRCUSTOPROD
            FROM
                PRODUTO     P
            WHERE
                P.CDPRODUTO = :P_CDPRODUTO
SQL;

        $params = array(
            "P_CDFILIAL" => $filial,
            "P_CDPRODUTO" => $produto,
            "P_DTLANC" => $dtLancamento
        );

        try{
            return $this->entityManager->getConnection()->fetchAssoc($sql, $params);
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function consultaCustoBase($produto, $cdprecobase){
        $sql = <<<SQL
        SELECT
                I.VRITEPREBASE AS  VRCUSTOPROD
            FROM
                ITEMPREBASE I
            WHERE
                (I.CDPRECOBASE = :P_CDPRECOBASE) AND
                (I.CDPRODUTO = :P_CDPRODUTO)
SQL;
        
        $params = array(
            "P_CDPRODUTO" => $produto,
            "P_CDPRECOBASE" => $cdprecobase
        );
    
        try{
            return $this->entityManager->getConnection()->fetchAssoc($sql, $params);
        } catch(\Exception $e) {
            throw $e;
        }
    }
}